package container

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

// TopResponse ContainerTopResponse
//
// Container "top" response.
// swagger:model TopResponse
type TopResponse struct {

	// Each process running in the container, where each process
	// is an array of values corresponding to the titles.
	Processes [][]string `json:"Processes"`

	// The ps column titles
	Titles []string `json:"Titles"`
}
