package dockerpre012

// This file contains a collection of methods that can be used from go-restful to
// generate Swagger API documentation for its models. Please read this PR for more
// information on the implementation: https://github.com/emicklei/go-restful/pull/215
//
// TODOs are ignored from the parser (e.g. TODO(andronat):... || TODO:...) if and only if
// they are on one line! For multiple line or blocks that you want to ignore use ---.
// Any context after a --- is ignored.
//
// Those methods can be generated by using hack/update-swagger-docs.sh

// AUTO-GENERATED FUNCTIONS START HERE
var map_Config = map[string]string{
	"": "Config is the list of configuration options used when creating a container. Config does not contain the options that are specific to starting a container on a given host.  Those are contained in HostConfig Exists only for legacy conversion, copy of type from fsouza/go-dockerclient",
}

func (Config) SwaggerDoc() map[string]string {
	return map_Config
}

var map_DockerConfig = map[string]string{
	"":       "DockerConfig is the list of configuration options used when creating a container.",
	"Labels": "This field is not supported in pre012 and will always be empty.",
}

func (DockerConfig) SwaggerDoc() map[string]string {
	return map_DockerConfig
}

var map_DockerImage = map[string]string{
	"": "DockerImage is for earlier versions of the Docker API (pre-012 to be specific). It is also the version of metadata that the container image registry uses to persist metadata.\n\nCompatibility level 4: No compatibility is provided, the API can change at any point for any reason. These capabilities should not be used by applications needing long term support.",
}

func (DockerImage) SwaggerDoc() map[string]string {
	return map_DockerImage
}

var map_ImagePre012 = map[string]string{
	"": "ImagePre012 serves the same purpose as the Image type except that it is for earlier versions of the Docker API (pre-012 to be specific) Exists only for legacy conversion, copy of type from fsouza/go-dockerclient",
}

func (ImagePre012) SwaggerDoc() map[string]string {
	return map_ImagePre012
}

var map_Mount = map[string]string{
	"": "Mount represents a mount point in the container.\n\nIt has been added in the version 1.20 of the Docker API, available since Docker 1.8. Exists only for legacy conversion, copy of type from fsouza/go-dockerclient",
}

func (Mount) SwaggerDoc() map[string]string {
	return map_Mount
}

// AUTO-GENERATED FUNCTIONS END HERE
