package container

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

// ChangeType Kind of change
//
// Can be one of:
//
// - `0`: Modified ("C")
// - `1`: Added ("A")
// - `2`: Deleted ("D")
//
// swagger:model ChangeType
type ChangeType uint8
