// Package libyaml contains internal helpers for working with YAML
//
// It's a reworked version of the original libyaml package from go-yaml v2/v3,
// adapted to work with Go specifications
package libyaml
